@@ -71,6 +71,34 @@ namespace larcv3 {
     return ss.str();
   }
 
+  template<size_t dimension>
+  larcv3::BBoxCollection<dimension> BBoxCollection<dimension>::compress(size_t compression, PoolType_t pool_type) const
+  {
+
+    std::array<size_t, dimension> comp;
+    for (size_t i = 0; i < dimension; ++i ) comp[i] = compression;
+
+    return this->compress(comp, pool_type);
+  }
+
+  template<size_t dimension>
+  larcv3::BBoxCollection<dimension> BBoxCollection<dimension>::compress(std::array<size_t, dimension> compression, PoolType_t pool_type) const
+  {
+    ImageMeta<dimension> compressed_meta = this->meta().compress(compression);
+
+    BBoxCollection<dimension> output(compressed_meta);
+
+    // Centroid and legth are in absolute coordinates, so not affected by downsampling
+    for (size_t i = 0; i < this->size(); i++) {
+      BBox<dimension> bbox(this->bbox(i).centroid(),
+                           this->bbox(i).half_length(),
+                           this->bbox(i).rotation_matrix());
+      output.append(bbox);
+    }
+
+    return output;
+  }
+
 template class BBox<2>;
 template class BBox<3>;
 
@@ -94,29 +122,42 @@ template<size_t dimension>
 void init_bbox_instance(pybind11::module m){
     using Class = larcv3::BBox<dimension>;
     pybind11::class_<Class> bbox(m, larcv3::as_string<Class>().c_str());
-    bbox.def(pybind11::init<>());
+
+    bbox.doc() = R"pbdoc(
+      Bounding Box 
+      ------------
+
+      Represents a bounding box in N dimensions, defined by a centroid and half_length in each axis from the centroid.
+      The rotation component is currently not used but provided in the data store implementation for future use.
+      Used to represent, for example, the location of a particle interaction, a bounding box for an instance of an object,
+      etc.
+    )pbdoc";
+
+    bbox.def(pybind11::init<>(), "Default Constructor");
     bbox.def(pybind11::init<const std::array<double, dimension>&, 
                             const std::array<double, dimension>&,
                             const std::array<double, dimension*dimension>
                            > (),
              pybind11::arg("centroid"),
              pybind11::arg("half_length"),
-             pybind11::arg("rotation")
+             pybind11::arg("rotation"),
+             "Constructor using centroid, half_length, and rotation."
     );
     bbox.def(pybind11::init<const std::array<double, dimension>&, 
                             const std::array<double, dimension>&
                            > (),
              pybind11::arg("centroid"),
-             pybind11::arg("half_length")
+             pybind11::arg("half_length"),
+             "Constructor using centroid and half_length."
     );
 
-    bbox.def("centroid",           &Class::centroid);
-    bbox.def("half_length",        &Class::half_length);
-    bbox.def("rotation_matrix",    &Class::rotation_matrix);
-    bbox.def("identity_rotation",  &Class::identity_rotation);
+    bbox.def("centroid",           &Class::centroid, "Get the value of the centroid.");
+    bbox.def("half_length",        &Class::half_length, "Get the value of the half_length.");
+    bbox.def("rotation_matrix",    &Class::rotation_matrix, "Get the value of the rotation matrix.");
+    bbox.def("identity_rotation",  &Class::identity_rotation, "Get the value of the identity rotation.");
 
-    bbox.def("dump",               &Class::dump);
-    bbox.def("__repr__",           &Class::dump);
+    bbox.def("dump",               &Class::dump, "Return a string representation of the BBox");
+    bbox.def("__repr__",           &Class::dump, "Return a string representation of the BBox");
 
 }
 
@@ -126,6 +167,14 @@ void init_bbox_collection(pybind11::module m){
     pybind11::class_<Class> bbox_c(m, larcv3::as_string<Class>().c_str());
     bbox_c.def(pybind11::init<>());
 
+    bbox_c.doc() = R"pbdoc(
+      Bounding Box Collection
+      ------------------------
+
+      A collection of Bounding Boxes in N dimensions.  Generally used to represent a sequence of Bounding
+      Boxes all within the same image.
+    )pbdoc";
+
     bbox_c.def("bbox",           &Class::bbox);
     bbox_c.def("as_vector",      &Class::as_vector);
     bbox_c.def("clear_data",     &Class::clear_data);
@@ -133,6 +182,11 @@ void init_bbox_collection(pybind11::module m){
     bbox_c.def("writeable_bbox", &Class::writeable_bbox);
     bbox_c.def("append",         &Class::append);
     bbox_c.def("size",           &Class::size);
+    bbox_c.def("meta", (const larcv3::ImageMeta<dimension>& (Class::*)() const )(&Class::meta), 
+      pybind11::return_value_policy::reference);
+    bbox_c.def("meta", (void (Class::*)(const larcv3::ImageMeta<dimension>& )  )(&Class::meta), 
+      pybind11::arg("meta"), 
+      pybind11::return_value_policy::reference);
 
 }
 
