@@ -16,7 +16,6 @@
 
 namespace larcv3 {
 
-  /// Global larcv3::SBClusterFactory to register ClusterAlgoFactory
   static EventTensorFactory<1> __global_EventTensor1DFactory__;
   static EventTensorFactory<2> __global_EventTensor2DFactory__;
   static EventTensorFactory<3> __global_EventTensor3DFactory__;
@@ -73,6 +72,9 @@ namespace larcv3 {
   }
 
 
+
+
+
   template<size_t dimension>
   void EventTensor<dimension>::open_in_datasets(hid_t group){
 
@@ -192,7 +194,6 @@ namespace larcv3 {
       dapl                          // hid_t dapl_id IN: Dataset access property list
     );
 
-    std::cout << "Created extents " << std::endl;
     /////////////////////////////////////////////////////////
     // Create the image_extents dataset
     /////////////////////////////////////////////////////////
@@ -224,7 +225,6 @@ namespace larcv3 {
       image_extents_cparms,               // hid_t dcpl_id IN: Dataset creation property list
       dapl                                // hid_t dapl_id IN: Dataset access property list
     );
-    std::cout << "Created image extents " << std::endl;
 
     /////////////////////////////////////////////////////////
     // Create the image_meta dataset
@@ -263,7 +263,6 @@ namespace larcv3 {
 
     _compression = compression;
 
-    std::cout << "Initialized" << std::endl;
 
     return;
   }
@@ -278,6 +277,7 @@ namespace larcv3 {
     // Create the Image dataset
     /////////////////////////////////////////////////////////
     // if ( ! group -> nameExists("images")){
+
     if (  get_num_objects(group) != 4){
         // std::cout << "Images dataset does not yet exist, creating it." << std::endl;
         // An image is stored as a flat vector, so it's type is float.
@@ -297,6 +297,8 @@ namespace larcv3 {
             chunk_size += image.size();
         }
 
+
+
         /*
          * Modify dataset creation properties, i.e. enable chunking.
          */
@@ -313,20 +315,20 @@ namespace larcv3 {
 
         // Create the extents dataset:
         H5Dcreate(
-          group,                           // hid_t loc_id  IN: Location identifier
+          group,                       // hid_t loc_id  IN: Location identifier
           "images",                    // const char *name      IN: Dataset name
           _data_types[IMAGES_DATASET], // hid_t dtype_id  IN: Datatype identifier
-          image_dataspace,            // hid_t space_id  IN: Dataspace identifier
-          lcpl,                            // hid_t lcpl_id IN: Link creation property list
-          image_cparms,               // hid_t dcpl_id IN: Dataset creation property list
-          dapl                             // hid_t dapl_id IN: Dataset access property list
+          image_dataspace,             // hid_t space_id  IN: Dataspace identifier
+          lcpl,                        // hid_t lcpl_id IN: Link creation property list
+          image_cparms,                // hid_t dcpl_id IN: Dataset creation property list
+          dapl                         // hid_t dapl_id IN: Dataset access property list
         );
 
 
     }
 
-    open_out_datasets(group);
 
+    open_out_datasets(group);
 
     // Serialization of images proceeds as:
     // 1) Read the current dimensions of all tables (extents, image_extents, image_meta, images)
