@@ -1,14 +1,14 @@
 /**
  * \file Voxel.h
  *
- * \ingroup core_DataFormat
+ * \ingroup DataFormat
  *
  * \brief Class def header for a class larcv3::Voxel
  *
- * @author kazuhiro
+ * @author cadams, kazuhiro
  */
 
-/** \addtogroup core_DataFormat
+/** \addtogroup DataFormat
 
     @{*/
 #ifndef __LARCV3DATAFORMAT_VOXEL_H
@@ -49,6 +49,7 @@ namespace larcv3 {
     //
     // uniry operators
     //
+    /// Add 
     inline Voxel& operator += (float value)
     { _value += value; return (*this); }
     inline Voxel& operator -= (float value)
@@ -224,6 +225,11 @@ namespace larcv3 {
     /// InstanceID_t setter
     inline void id(const InstanceID_t id) { _id = id; }
 
+#ifdef LARCV_INTERNAL
+
+    void set(pybind11::array_t<size_t> indexes, pybind11::array_t<float> values);
+#endif
+
     //
     // Uniry operations
     //
@@ -236,6 +242,8 @@ namespace larcv3 {
     inline VoxelSet& operator /= (float factor)
     { for(auto& vox : _voxel_v) vox /= factor; return (*this); }
 
+    std::vector<larcv3::Voxel> & writeable_voxel_vector(){return _voxel_v;}
+
   protected:
     /// Instance ID
     InstanceID_t _id;
@@ -329,9 +337,10 @@ namespace larcv3 {
     /// Access ImageMeta of specific projection
     inline const larcv3::ImageMeta<dimension>& meta() const { return _meta; }
 
-
+#ifdef LARCV_INTERNAL
     // Take this sparseTensor and return it as a dense numpy array
     pybind11::array_t<float> dense();
+#endif
 
     larcv3::Tensor<dimension> to_tensor();
 
